//
//  CPDFViewController.m
//  PageEdit-Ctrl-Demo
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFViewController.h"

#import <ComPDFKit/ComPDFKit.h>
#import <compdfkit_tools/compdfkit_tools.h>
 
@interface CPDFViewController () <CPDFPageEditViewControllerDelegate, CPDFSignatureViewControllerDelegate>

@property (nonatomic, strong) CPDFSignatureWidgetAnnotation * signatureAnnotation;

@end

@implementation CPDFViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.view.backgroundColor = [CPDFColorUtils CPDFViewControllerBackgroundColor];
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    
    if([self.popMenu superview]) {
        if (@available(iOS 11.0, *)) {
            [self.popMenu showMenuInRect:CGRectMake(self.view.frame.size.width - self.view.safeAreaInsets.right - 250, CGRectGetMaxY(self.navigationController.navigationBar.frame), 250, 200)];
        } else {
            // Fallback on earlier versions
            [self.popMenu showMenuInRect:CGRectMake(self.view.frame.size.width - 250, CGRectGetMaxY(self.navigationController.navigationBar.frame), 250, 200)];
        }
    }
}

#pragma mark - Private Mehtods

- (void)initWitNavigationTitle {
    CNavigationBarTitleButton * navTitleButton = [[CNavigationBarTitleButton alloc] init];
    self.titleButton = navTitleButton;
    self.navigationTitle = NSLocalizedString(@"Page Edit", nil);
    [navTitleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    [navTitleButton setTitleColor:[CPDFColorUtils CAnyReverseBackgooundColor] forState:UIControlStateNormal];
    self.titleButton.frame = CGRectMake(0, 0, 100, 30);
    self.navigationItem.titleView = self.titleButton;
}

#pragma mark - Action

- (void)buttonItemClicked_thumbnail:(id)sender {
    CPDFPageEditViewController *pageEditViewcontroller = [[CPDFPageEditViewController alloc] initWithPDFView:self.pdfListView];
    pageEditViewcontroller.pageEditDelegate = self;
    pageEditViewcontroller.modalPresentationStyle = UIModalPresentationFullScreen;
    [self.navigationController presentViewController:pageEditViewcontroller animated:YES completion:nil];
//    [self.navigationController pushViewController:pageEditViewcontroller animated:NO];
}

- (void)titleButtonClickd:(id)sender {
    
}

#pragma mark - CPDFPageEditViewControllerDelegate

- (void)pageEditViewControllerDone:(CPDFPageEditViewController *)pageEditViewController {
    __weak typeof(self) weakSelf = self;
    [weakSelf reloadDocumentWithFilePath:self.filePath password:nil completion:^(BOOL result) {
        [weakSelf.pdfListView reloadInputViews];
    }];
    
    [weakSelf.pdfListView reloadInputViews];
}

- (void)pageEditViewController:(CPDFPageEditViewController *)pageEditViewController pageIndex:(NSInteger)pageIndex {
    [self.pdfListView goToPageIndex:pageIndex animated:NO];
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - CPDFListViewDelegate

- (void)PDFListViewPerformTouchEnded:(CPDFListView *)pdfView {
    CGFloat tPosY = 0;
    if (self.navigationController.navigationBarHidden) {
        [self.navigationController setNavigationBarHidden:NO animated:YES];
        [UIView animateWithDuration:0.3 animations:^{
            self.pdfListView.pageSliderView.alpha = 1.0;
        }];
        CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
        tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;

    } else {
        [self.navigationController setNavigationBarHidden:YES animated:YES];
        [UIView animateWithDuration:0.3 animations:^{
            self.pdfListView.pageSliderView.alpha = 0.0;
        }];
    }
}

- (void)PDFListViewPerformSignatureWidget:(CPDFListView *)pdfView forAnnotation:(CPDFSignatureWidgetAnnotation *)annotation {
    self.signatureAnnotation = annotation;
    AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
    CPDFSignatureViewController *signatureVC = [[CPDFSignatureViewController alloc] initWithStyle:nil];
    presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:signatureVC presentingViewController:self];
    signatureVC.delegate = self;
    signatureVC.transitioningDelegate = presentationController;
    [self presentViewController:signatureVC animated:YES completion:nil];
}

#pragma mark - CPDFSignatureViewControllerDelegate

- (void)signatureViewControllerDismiss:(CPDFSignatureViewController *)signatureViewController {
    self.signatureAnnotation = nil;
}

- (void)signatureViewController:(CPDFSignatureViewController *)signatureViewController image:(UIImage *)image {
    if(self.signatureAnnotation) {
        [self.signatureAnnotation signWithImage:image];
        [self.pdfListView setNeedsDisplayForPage:self.signatureAnnotation.page];
        self.signatureAnnotation = nil;
    }
}

@end
